#! /usr/bin/env bash
#
# Copyright 2021 Xilinx, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
set -x
set -e
source locate.sh
L_CURLY='{'
R_CURLY='}'
BIF="my.bif"

rm -f ${BIF}

source ${MY_DIR}/decode_design_dir.sh $1
BUILD_DIR="${DESIGN_DIR}/bootgen"
LOCAL_PDI=$(basename "${DESIGN_PDI}")
cat << EOF1 >>${BIF}
new_bif:
${L_CURLY}
    image
    ${L_CURLY}
        ${L_CURLY}
            type = bootimage
            file = ${LOCAL_PDI}
        ${R_CURLY}
    ${R_CURLY}
EOF1

source ${MY_DIR}/decode_app_dir.sh $2
source ${MY_DIR}/decode_build.sh
ELF1="${APP_R5_ELF}"
CORE_1="${APP_R5__CORE}"
APP_ELF1="${APP_NAME}-${CORE_1}.elf"
BUILD_DIR="${BUILD_DIR}-${APP_NAME}-${APP_R5__CORE}"
BIF1="${BIF}1"
rm -f "${BIF1}"
cat << EOF2 >${BIF1}
    image
    ${L_CURLY}
        name = lpd
        partition
        ${L_CURLY}
            core = ${APP_R5_CORE}
            file = ${APP_ELF1}
        ${R_CURLY}
    ${R_CURLY}
EOF2

if [[ "X$3" != "X" ]] ; then
    source ${MY_DIR}/decode_app_dir.sh $3
    source ${MY_DIR}/decode_build.sh
    ELF2="${APP_R5_ELF}"
    CORE_2="${APP_R5__CORE}"
    APP_ELF2="${APP_NAME}-${CORE_2}.elf"
    BUILD_DIR="${BUILD_DIR}-${APP_NAME}-${APP_R5__CORE}"
    BIF2="${BIF}2"
    rm -f "${BIF2}"
    cat << EOF3 >${BIF2}
    image
    ${L_CURLY}
        name = lpd
        partition
        ${L_CURLY}
            core = ${APP_R5_CORE}
            file = ${APP_ELF2}
        ${R_CURLY}
    ${R_CURLY}
EOF3
else
    ELF2=""
    CORE_2=""
    APP_ELF2=""
fi

if [[ "X${CORE_2}" != "X" ]]; then
    if [[ "X${CORE_1}" = "X${CORE_2}" ]]; then
        echo "Error: Both elf files use the same core ${CORE_1}"
        exit 1
    fi
    if [[ ${CORE_1} = "r5_0" ]]; then
        cat ${BIF1} >> ${BIF}
        cat ${BIF2} >> ${BIF}
    else
        cat ${BIF2} >> ${BIF}
        cat ${BIF1} >> ${BIF}
    fi
    rm -f ${BIF1}
    rm -f ${BIF2}
else
    cat ${BIF1} >> ${BIF}
    rm -f ${BIF1}
fi

BIN="$(echo -n "${BUILD_DIR}/"; echo -n ${BUILD_DIR}|sed -e 's:.*/bootgen-::';echo '.bin')"
cat << EOF4 >>${BIF}
${R_CURLY}
EOF4

rm -rf ${BUILD_DIR}
mkdir -p ${BUILD_DIR}
mv ${BIF} ${BUILD_DIR}
pushd ${BUILD_DIR}
    cp "${ELF1}" "${APP_ELF1}"
    cp "${DESIGN_PDI}" "${LOCAL_PDI}"
    if [[ "X${ELF2}" != "X" ]]; then
        cp "${ELF2}" "${APP_ELF2}"
    fi
    bootgen \
            -arch versal \
            -fill 0xff \
            -image ${BUILD_DIR}/${BIF} \
            -log trace \
            -w on \
            -o "${BIN}"
popd

exit 0
