#! /usr/bin/perl -w
#
# Copyright 2021 Xilinx, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

use strict;
use IO::Socket::INET;

if($#ARGV != 1) {
    die "Usage: $0 host elf"
}
my $host = $ARGV[0];
my $elf = $ARGV[1];
my $core = -1;
-f "$elf"  or die "'$elf' -> does not exist\n";
if(0) {
    ;
} elsif($elf =~ /-r5-0.elf$/) {
    $core = 0;
} elsif($elf =~ /-r5-1.elf$/) {
    $core = 1;
} else {
    die("Unknown core for $elf\n");
}

open(my $dis,"armr5-none-eabi-objdump -d \"$elf\"|") or die("dis\n");
my $oldoff = -1;
my $oldsym = '';
my $off;
my $sym; 
my %Syms = ();
while(<$dis>) {
    chomp;
    if(/^([0-9a-zA-Z]+)\s+<([a-zA-Z_][a-zA-Z0-9_]+)>:/) {
        $off = hex($1);
        $sym = $2;
        if($oldoff >= 0) {
            for(my $p = 0, my $o = $oldoff; $o < $off; $p += 4, $o += 4) {
                $Syms{$o} = $oldsym . "+" . "$p";
            }
        }
        $oldoff = $off;
        $oldsym = $sym;
    }
}
if($oldoff >= 0) {
    for(my $p = 0, my $o = $oldoff; $o < $off; $p += 4, $o += 4) {
        $Syms{$o} = $oldsym . "+" . "$p";
    }
}
close($dis);

if(0) {
    my $key;
    my $val;

    foreach $key (sort({$a <=> $b} keys(%Syms))) {
        $val = $Syms{$key};
        print("$key $val\n");
    }
}

my $localhost;
my $localport;

open(my $fh,'-|',"xsdb -eval 'xsdbserver start' < /dev/null") or die("xsdb\n");
while(<$fh>) {
    #Connect to this XSDB server use host XilinxVboxVM180402 and port 44895
    chomp;
    if(/^Connect to this XSDB server use host ([^\s]+) and port ([0-9]+)/) {
        $localhost = $1;
        $localport = $2;
        last;
    }
}
print "Connecting to $localhost $localport\n";

my $client = IO::Socket::INET->new(
            PeerAddr => $localhost,
            PeerPort => $localport,
            Proto => 'tcp'
        ) or die("sock\n");

$client->autoflush();

my $command;
my $resp;
$command = "connect -host $host\n";
$client->send($command) or die($command);
#print "-> $command";
$client->recv($resp,1024);
#print "<- $resp";

if($resp !~ /error/) {
    $command = "targets -set -filter {name =~ \"Cortex-R5 #${core}\"}\n";
    $client->send($command) or die($command);
    #print "-> $command";
    $client->recv($resp,1024);
    #print "<- $resp";
}
if($resp !~ /error/) {
    $command = "targets\n";
    $client->send($command) or die($command);
    #print "-> $command";
    $client->recv($resp,1024);
    #    my @Things = split('\n',$resp);
    #foreach my $Thing (@Things) {
    #    print "$Thing\n";
    #}
}

if($resp !~ /error/) {
    $command = "rrd pc\n";
    while($resp !~ /error/) {
        $client->send($command) or die($command);
        $client->recv($resp,1024);
        #        print "-> $command";
        #print "<- $resp";
        chomp $resp;
        if($resp =~ /^okay pc:\s*([0-9a-fA-F]+)/) {
            $off = hex($1);
            my $where = $Syms{$off};
            if(defined($where)) {
                print "$Syms{$off}\n";
            } else {
                print "$off(????)\n";
            }
        } else {
            print $resp;
        }
    }
}
exit(0);
