/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef X86_CSV_H__
#define X86_CSV_H__

struct CsvRow;
struct ImageInfo;
struct BoardInfo;

typedef struct CsvRow CsvRow;
typedef struct ImageInfo ImageInfo;
typedef struct BoardInfo BoardInfo;

/*
 * The maximum number of tokens for any command including the command itself
 */
#define MAX_TOKENS  16

/*
 * Each line of a CSV file is captured and tokenized in this structure
 */
#define CSV_LINE_BYTES  4096
#define CSV_MAX_TOKENS  MAX_TOKENS
struct CsvRow {
    int iLine;
    char pcLine[CSV_LINE_BYTES];
    int iNumTokens;
    char* ppcTokens[CSV_MAX_TOKENS + 1];
};

typedef enum {
    IMAGE_TYPE_PROM,
    IMAGE_TYPE_STAGE1,
    IMAGE_TYPE_STAGE2
} IMAGE_TYPE;

/*
 * Internal representation of an image csv row
 * String tokens that do not need extra parsing are accessed directly
 */
#define II_FULLNAME_BYTES  4096
#define II_ID_BYTES        128
struct ImageInfo {
    ListElem tListElem;

    IMAGE_TYPE eType;
    uint32_t qChecksum;
    char pcFullName[II_FULLNAME_BYTES];
    uint32_t qSize;
    void* pxMap;

    CsvRow tCR;
};

/*
 * Internal representation of a board csv row
 * String tokens that do not need extra parsing are accessed directly
 */
struct BoardInfo {
    ListElem tListElem;

    uint32_t qR5CommBar;
    uint32_t qTCMOffset;
    uint32_t qConnectionMode;    /* CONNECTION_MODE enum */
    uint32_t qBusWidth;         /* 1 / 2 / 4 */
    uint32_t qQspiPeripheralInputSpeedHz;
    uint32_t qQspiBusMaxSpeedHz;
    uint32_t qCacheCoherentDMA;
    uint32_t qPEBSize;
    uint32_t qNumPEBs;
    uint32_t qSubsectorSize;
    uint32_t qPEBOffsetB;
    uint32_t qPEBOffsetG;

    uint32_t qPrescaler;

    CsvRow tCR;
};

extern bool ParseImagesCsv(Sys* pSys,ListHead* pListHead);
extern bool ParseBoardsCsv(Sys* pSys,ListHead* pListHead);

#endif  /* X86_CSV_H__ */
