/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <x86_app.h>

bool CheckBoardTypeSyntax(Sys* pSys,const char* pcFile,int iLine,char* pcBoardType)
{
    char* pc;
    bool yValid;
    
    for(pc = pcBoardType; *pc; pc++) {
        *pc = (char)tolower(*pc);
    }
    yValid = true;
    if(0
            || ('_' == pcBoardType[0])
            || (('a' < pcBoardType[0]) && (pcBoardType[0] < 'z'))
      ) {
        for(pc = &pcBoardType[1]; *pc; pc++) {
            if(0
                    || ('_' == *pc)
                    || (('a' <= *pc) && (*pc <= 'z'))
                    || (('0' <= *pc) && (*pc <= '9'))
              ) {
                ;
            } else {
                yValid = false;
                break;
            }
        }
    } else {
        yValid = false;
    }
    if(!yValid) {
        Die(pSys,0,"%s:%d Invalid board type '%s': Not a valid C identifier",
                pcFile,
                iLine,
                pcBoardType
           );
    }
    return yValid;
}

static bool CheckBDFSyntax(Sys* pSys,const char* pcFile,int iLine,BoardInfo* pBI,char* pcBDF)
{
    char c;
    int iNumTokens;
    char* ppcTokens[5];
    int iFirstToken;

    if(0 == strcmp(pBI->tCR.ppcTokens[0],"sim")) {
        /* "BDF" is really MultibootChar:FlashFile */
        switch((c = *pcBDF++)) {
        case 'a':   /* FallThrough */
        case 'A':   /* FallThrough */
        case 'b':   /* FallThrough */
        case 'B':   /* FallThrough */
        case 'g':   /* FallThrough */
        case 'G':   /* FallThrough */
            pSys->qMultibootChar = (uint32_t)tolower(c);
            break;

        default:
            Die(pSys,0,"%s:%d Invalid multiboot char '%c' in BoardType '%s' BDF '%s'",
                    pcFile,
                    iLine,
                    c,
                    pBI->tCR.ppcTokens[0],
                    pcBDF - 1
               );
            /* Not reached */
            return false;
        }
        if(*pcBDF++ != ':') {
            Die(pSys,0,"%s:%d Need ':' after multiboot char in BoardType '%s' BDF '%s'",
                    pcFile,
                    iLine,
                    pBI->tCR.ppcTokens[0],
                    pcBDF
               );
            /* Not reached */
            return false;
        }
        pSys->pcSimFlash = pcBDF;
    } else {
        pSys->pcSimFlash = 0;
        iNumTokens = Tokenize(
                pSys,
                pcFile,
                iLine,
                pcBDF,
                ":.\n",
                4,
                ppcTokens
                );
        if(iNumTokens < 0) {
            return false;
        }
        iFirstToken = 0;
        if(0) {
            ;
        } else if(1 == iNumTokens) {
            pSys->tBDF.qDomain = 0;
            pSys->tBDF.qBoard = 0;
            pSys->tBDF.qDevice = 0;
            pSys->tBDF.qFunction = 0;
            pSys->tBDF.pcUioFile = ppcTokens[0];
        } else {
            pSys->tBDF.pcUioFile = 0;
            if(4 == iNumTokens) {
                if(!GetBaseNumber(
                        pSys,
                        pcFile,
                        iLine,
                        "qDomain",
                        ppcTokens[iFirstToken++],
                        16,
                        &pSys->tBDF.qDomain
                        )
                   ) {
                    return false;
                }
            } else {
                pSys->tBDF.qDomain = 0;
            }
            if(0
                    || !GetBaseNumber(
                            pSys,
                            pcFile,
                            iLine,
                            "qBoard",
                            ppcTokens[iFirstToken + 0],
                            16,
                            &pSys->tBDF.qBoard
                            )
                    || !GetBaseNumber(
                            pSys,
                            pcFile,
                            iLine,
                            "qDevice",
                            ppcTokens[iFirstToken + 1],
                            16,
                            &pSys->tBDF.qDevice
                            )
                    || !GetBaseNumber(
                            pSys,
                            pcFile,
                            iLine,
                            "qFunction",
                            ppcTokens[iFirstToken + 2],
                            16,
                            &pSys->tBDF.qFunction
                            )
                    ) {
                return false;
            }
            /* /sys/bus/pci/devices/0000\:01\:00.0/enable */
            pSys->tBDF.pcEnableFile[BDF_SYSFILE_BYTES - 1] = 0;
            (void)snprintf(pSys->tBDF.pcEnableFile,BDF_SYSFILE_BYTES,
                    "/sys/bus/pci/devices/%04x:%02x:%02x.%1x/enable",
                    pSys->tBDF.qDomain,
                    pSys->tBDF.qBoard,
                    pSys->tBDF.qDevice,
                    pSys->tBDF.qFunction
                    );
            if(0 != pSys->tBDF.pcEnableFile[BDF_SYSFILE_BYTES - 1]) {
                Die(pSys,0,"Internal error: Insufficient BDF file space %d\n",BDF_SYSFILE_BYTES);
                /* Not reached */
                return false;
            }

            /* /sys/bus/pci/devices/0000\:01\:00.0/resourceN */
            pSys->tBDF.pcResourceFile[BDF_SYSFILE_BYTES - 1] = 0;
            (void)snprintf(pSys->tBDF.pcResourceFile,BDF_SYSFILE_BYTES,
                    "/sys/bus/pci/devices/%04x:%02x:%02x.%1x/resource%1x",
                    pSys->tBDF.qDomain,
                    pSys->tBDF.qBoard,
                    pSys->tBDF.qDevice,
                    pSys->tBDF.qFunction,
                    pBI->qR5CommBar
                    );
            if(0 != pSys->tBDF.pcResourceFile[BDF_SYSFILE_BYTES - 1]) {
                Die(pSys,0,"Internal error: Insufficient BDF file space %d\n",BDF_SYSFILE_BYTES);
                /* Not reached */
                return false;
            }
        }
    }
    return true;
}

bool SearchBoardType(ListHead* pListHead,ListElem* pListElem,void* pxArg)
{
    BoardInfo* pBI = (BoardInfo*)pListElem;

    if(0 == strcmp(pBI->tCR.ppcTokens[0],(char*)pxArg)) {
        return true;
    }
    return false;
}

bool OpenDevice(Sys* pSys,const char* pcFile,int iLine,char* pcDevice)
{
    char* pcBoardType;
    char* pcBDF;
    BoardInfo* pBI = 0;
    bool yRet = true;
    uint32_t qErrCode;
    int iRBE;
    uint32_t qErrBSP;
    char pcErrString[ERR_STRING_BUFSIZE];

    pcBDF = strchr(pcDevice,':');
    if(!pcBDF || !*(pcBDF + 1)) {
        Die(pSys,0,"%s: Invalid BoardTypeBDF syntax",pSys->pcDevice);
        /* Not reached */
        return false;
    }
    *pcBDF++ = 0;
    pcBoardType = pcDevice;

    if(1
            && CheckBoardTypeSyntax(pSys,pcFile,iLine,pcBoardType)
            && ((pBI = (BoardInfo*)CheckElems(&pSys->tBoardsCsvListHead,SearchBoardType,pcBoardType)))
            && CheckBDFSyntax(pSys,pcFile,iLine,pBI,pcBDF)
      ) {
        pSys->pcDevice = pcDevice;
        pSys->pcBoardType = pcBoardType;
        pSys->pcBDF = pcBDF;
        pSys->pBI = pBI;
        pSys->pbReserved = malloc(pSys->pBI->qSubsectorSize * 3);
        if(!pSys->pbReserved) {
            Die(pSys,errno,"malloc");
            /* Not reached */
            return false;
        }
        for(iRBE = 0; iRBE < 3; iRBE++) {
            pSys->ppRBE[iRBE] = (ReservedBlockEntry*)&pSys->pbReserved[iRBE * pSys->pBI->qSubsectorSize];
        }
    }
    if(!pBI) {
        pSys->pcDevice = 0;
        pSys->pcBoardType = 0;
        pSys->pcBDF = 0;
        pSys->pBI = 0;
        pSys->pbReserved = 0;
        pSys->ppRBE[0] = 0;
        pSys->ppRBE[1] = 0;
        pSys->ppRBE[2] = 0;
        Die(pSys,0,"%s: No such board type in database",pcBoardType);
        /* Not reached */
        return false;
    }

    if(0 == strcmp(pSys->pcBoardType,"sim")) {
        yRet &= OpenSim(pSys);
    } else {
        yRet &= OpenR5(pSys);
    }

    yRet &= WaitForSyncWord(pSys,TIMEOUT_FEEDC0DE);

    if(yRet) {
        qErrCode = VerifyFlash(
                pSys,
                pSys->pBI->qConnectionMode,
                pSys->pBI->qBusWidth,
                pSys->pBI->qQspiPeripheralInputSpeedHz,
                pSys->pBI->qPrescaler,
                pSys->pBI->qCacheCoherentDMA,
                pSys->pBI->qPEBSize,
                pSys->pBI->qNumPEBs,
                pSys->pBI->qSubsectorSize,
                &qErrBSP
                );
        if(qErrCode != ERR_NONE) {
            Die(pSys,0,"VerifyFlash(%s): Error %s (BSP error %d)",
                    pSys->pcBoardType,
                    GetErrString((ERR)qErrCode,pcErrString),
                    qErrBSP
                    );
            yRet = false;
        }
    }
    (void)DecodeDevInfo(pSys);
    return yRet;
}

bool CloseDevice(Sys* pSys)
{
    bool yRet = true;

    if(pSys->pbReserved) {
        free(pSys->pbReserved);
        pSys->pbReserved = 0;
        pSys->ppRBE[0] = 0;
        pSys->ppRBE[1] = 0;
        pSys->ppRBE[2] = 0;
    }
    if(pSys->pcBoardType) {
        if(0 == strcmp(pSys->pcBoardType,"sim")) {
            yRet &= CloseSim(pSys);
        } else {
            yRet &= CloseR5(pSys);
        }
        pSys->pcDevice = 0;
        pSys->pcBoardType = 0;
        pSys->pcBDF = 0;
        pSys->pBI = 0;
    }
    return yRet;
}
