/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <x86_app.h>

bool OpenR5(Sys* pSys)
{
    R5Data* pR5Data;
    int iFile;
    
    pSys->pDevData = calloc(1,sizeof(R5Data));
    if(!pSys->pDevData) {
        Die(pSys,errno,"%s",__func__);
        /* Not reached */
        return false;
    }
    pR5Data = (R5Data*)pSys->pDevData;

    if(pSys->tBDF.pcUioFile) {
        pR5Data->qMapSize = R5_MAP_MIN_SIZE;
        if(!MapFile(
                pSys,                                   /* Sys* pSys, */
                __func__,                               /* const char* pcContext, */
                pSys->tBDF.pcUioFile,                   /* const char* pcFile, */
                MAPPING_UIO,                            /* MAPPING eMapping, */
                pSys->pBI->qR5CommBar * getpagesize(),  /* uint64_g gOffset */
                &pR5Data->qMapSize,                     /* uint32_t* pqRequiredSize, */
                &pR5Data->pxMap                         /* void** ppxMap */
                )) {
            free(pSys->pDevData);
            pSys->pDevData = 0;
            Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcUioFile);
            /* Not reached */
            return false;
        }
    } else {
        iFile = open(pSys->tBDF.pcEnableFile,O_WRONLY);
        if(iFile < 0) {
            Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
            /* Not reached */
            return false;
        }
        if(2 != write(iFile,"1\n",2)) {
            Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
            /* Not reached */
            return false;
        }
        if(0 != close(iFile)) {
            Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
            /* Not reached */
            return false;
        }

        if(!MapFile(
                pSys,                       /* Sys* pSys, */
                __func__,                   /* const char* pcContext, */
                pSys->tBDF.pcResourceFile,  /* const char* pcFile, */
                MAPPING_W_FIXED,            /* MAPPING eMapping, */
                0,                          /* uint64_t gOffset */
                &pR5Data->qMapSize,         /* uint32_t* pqRequiredSize, */
                &pR5Data->pxMap             /* void** ppxMap */
                )) {
            free(pSys->pDevData);
            pSys->pDevData = 0;
            Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcResourceFile);
            /* Not reached */
            return false;
        }
        if(pR5Data->qMapSize < R5_MAP_MIN_SIZE) {
            free(pSys->pDevData);
            pSys->pDevData = 0;
            Die(pSys,0,"%s(%s) must be at least %d MiB in size",
                    __func__,
                    pSys->tBDF.pcResourceFile,
                    R5_MAP_MIN_SIZE / (1024*1024)
                    );
            /* Not reached */
            return false;
        }
    }
    pSys->pMA = (MessageArea*)((uint8_t*)pR5Data->pxMap + pSys->pBI->qTCMOffset + R5_MESSAGE_AREA_OFFSET);
    pSys->pAA = (AuxArea*)((uint8_t*)pR5Data->pxMap + pSys->pBI->qTCMOffset + R5_AUX_AREA_OFFSET);

    return true;
}

bool CloseR5(Sys* pSys)
{
    bool yRet = true;
    R5Data* pR5Data;
    int iFile;

    pSys->pMA = 0;
    pSys->pAA =  0;

    pR5Data = (R5Data*)pSys->pDevData;

    if(UnmapFile(
            pSys,               /* Sys* pSys, */
            pR5Data->qMapSize,  /* uint32_t qSize */
            &pR5Data->pxMap     /* void* pxMap */
        )) {
        yRet = false;
    }

    iFile = open(pSys->tBDF.pcEnableFile,O_WRONLY);
    if(iFile < 0) {
        Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
        /* Not reached */
        return false;
    }
    if(2 != write(iFile,"0\n",2)) {
        Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
        /* Not reached */
        return false;
    }
    if(0 != close(iFile)) {
        Die(pSys,errno,"%s(%s)",__func__,pSys->tBDF.pcEnableFile);
        /* Not reached */
        return false;
    }

    (void)free(pSys->pDevData);
    pSys->pDevData = 0;

    return yRet;
}
